IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[qIISCreateJob]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[qIISCreateJob]
GO

-- =======================================================================================================================================
-- Author:			Philip Brassington
-- Creation date: 
-- Description:		Creates Job for Job Logging
-- Called By:		Used by Job Logging
-- Notes:
--
-- =======================================================================================================================================
-- History:
-- 13/09/2018 - Jack Foulkes - Added SQL to Stop Constraint Errors
-- =======================================================================================================================================
--

CREATE PROC [dbo].[qIISCreateJob] 
	@JobCode VARCHAR(50) OUTPUT,
	@Caller VARCHAR(255),
	@CallerTel VARCHAR(50),
	@CallerEmail VARCHAR(50),
	@CallerREF VARCHAR(50),
	@CallerSiteId VARCHAR(50),
	@CallerLocationId VARCHAR(50),
	@EquipmentId VARCHAR(50),
	@ReportedFault TEXT,
	@JobPriorityId VARCHAR(50),
	@JobTypeShortName VARCHAR(50),
	@UserId VARCHAR(50),
	@DeconCert VARCHAR(50)
AS
BEGIN
	--
	-- Returns:
	--	0 - Success, job created
	--	1 - Unrecognised @CallerSiteId. The job has been created
	--	2 - Unrecognised @CallerLocationId. The job has been created
	--	3 - Unrecognised @EquipmentId. The job has NOT been created
	--	4 - Unrecognised @JobPriorityId. The job has been created
	
	-- Constraint Fixes Start
	-- If database has constraints we cannot set an Id to the value of an empty string, must be NULL.
	-- If Location is provided but not site, try to fetch the site, otherwise set to NULL.

	IF ISNULL(@CallerLocationId, '') != '' AND @CallerSiteId = ''
	BEGIN

		-- Try Fetch SiteId
		SELECT TOP 1 @CallerSiteId = SiteId FROM [Location]
		WHERE LocationId = @CallerLocationId

	END

	--Check CallerSiteId
	IF ISNULL(@CallerSiteId, '') = ''
	BEGIN
		SET @CallerSiteId = NULL
	END

	--Check CallerLocationId
	IF ISNULL(@CallerLocationId, '') = ''
	BEGIN
		SET @CallerLocationId = NULL
	END


	-- Constraint Fixes End

	DECLARE @RetVal INT
	SET @RetVal = 0

	IF @CallerSiteId IS NOT NULL
	BEGIN
		IF NOT EXISTS (SELECT * FROM Site WHERE SiteId = @CallerSiteId)
			SET @RetVal = 1
	END

	IF @CallerLocationId IS NOT NULL
	BEGIN
		IF NOT EXISTS (SELECT * FROM Location WHERE LocationId = @CallerLocationId)
			SET @RetVal = 2
	END

	IF @JobPriorityId IS NOT NULL
	BEGIN
		IF NOT EXISTS (SELECT * FROM JobPriority WHERE JobPriorityId = @JobPriorityId)
			SET @RetVal = 4
	END

	IF @EquipmentId IS NOT NULL
	BEGIN
		IF NOT EXISTS (SELECT * FROM Equipment WHERE EquipmentId = @EquipmentId)
			RETURN 3
	END

	--
	-- Get the next Job No
	EXEC qNextCode 17, @JobCode OUTPUT
	
	--
	-- Pad the job number
	DECLARE @JobSort VARCHAR(50)
	IF ISNUMERIC(LTRIM(RTRIM(@JobCode))) != 0
	BEGIN
		SELECT 	@JobSort = SPACE( 50 - LEN(LTRIM(RTRIM(@JobCode)))) + LTRIM(RTRIM(@JobCode))
	END

	--
	-- Get the new job type
	DECLARE @JobTypeId VARCHAR(50)
	IF @JobTypeShortName IS NULL
	BEGIN
		SELECT TOP 1 @JobTypeId = JobTypeId
		FROM JobType
		WHERE JobTypeClassId = 'WEBCALL'
		ORDER BY JobTypeCode
	END
	ELSE
	BEGIN
		SELECT TOP 1 @JobTypeId = JobTypeId
		FROM JobType
		WHERE JobTypeShortName = @JobTypeShortName
		ORDER BY JobTypeCode
	END

	--
	-- Get the new job status
	DECLARE @JobStatusId VARCHAR(50)
	SELECT TOP 1 @JobStatusId = JobStatusId
	FROM JobStatus
	WHERE JobStatusClassId = 'NOTSTARTED'
	ORDER BY JobStatusCode

	--
	-- If we have equipment then brand, model & category must come from that
	DECLARE @BrandId VARCHAR(50)
	DECLARE @ModelId VARCHAR(50)
	DECLARE @CategoryId VARCHAR(50)
	DECLARE @TeamId VARCHAR(50)
	DECLARE @JobId VARCHAR(50)
	
	SET @BrandId = NULL
	SET @ModelId = NULL
	SET @CategoryId = NULL
	SET @TeamId = NULL
	SET @JobId = NEWID()

	IF @EquipmentId IS NOT NULL
	BEGIN
		SELECT @BrandId = BrandId, 
			@ModelId = ModelId, 
			@CategoryId = CategoryId,
			@TeamId = TeamId
		FROM Equipment
		WHERE EquipmentId = @EquipmentId
	END

	--
	-- Get the contract
	DECLARE @ContractId VARCHAR(50)
	SELECT @ContractId = dbo.CurrentContractId( @EquipmentId, GETDATE())

	--
	-- Get the customer contract
	DECLARE @CustomerContractId VARCHAR(50)
	SELECT @CustomerContractId = dbo.CurrentCustomerContractId( @EquipmentId, GETDATE())

	--
	-- Create the job
	INSERT INTO Job
		(JobId, JobCode, CallDate, Caller, CallerTel, CallerEMail, CallerRef,CallerSiteId, CallerLocationId,
		JobPriorityId, EquipmentId, CreationDate, JobSort, TeamId,
		BrandId, ModelId, CategoryId, ReportedFault, JobTypeId, JobStatusId,
		ContractId, CustomerContractId, DecontamCertNo)
	VALUES
		(@JobId, @JobCode, GETDATE(), @Caller, @CallerTel, @CallerEmail, @CallerREF, @CallerSiteId, @CallerLocationId,
		@JobPriorityId, @EquipmentId, GETDATE(), @JobSort, @TeamId, @BrandId, 
		@ModelId, @CategoryId, @ReportedFault, @JobTypeId, @JobStatusId, @ContractId,
		@CustomerContractId, @DeconCert)
	
	--
	-- Audit the job creation
	INSERT INTO CommonAudit
		( AuditId, AuditType, AuditDate, ChangedById, OriginalValue, NewValue, EntityId, AuditAction )
	VALUES ( NEWID(), 'JBC', GETDATE(), @UserId, '', 'Job Created', @JobId, 'Insert' )

	--
	-- Audit the status
	IF @JobStatusId IS NOT NULL
	BEGIN
		DECLARE @AuditDesc2 VARCHAR(255)
		SELECT @AuditDesc2 = ISNULL(JobStatusShortName, '')
		FROM JobStatus
		WHERE JobStatusId = @JobStatusId

		INSERT INTO CommonAudit
			( AuditId, AuditType, AuditDate, ChangedById, OriginalValue, NewValue, EntityId, AuditAction )
		VALUES ( NEWID(), 'JST', GETDATE(), @UserId, NULL, @AuditDesc2, @JobId, 'Insert' )
	END


	--
	-- Done
	RETURN @RetVal

END













